import { getData } from './getData';
import { select, pie, arc, scaleOrdinal, format } from 'd3';

const svg = select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');
const xAxisTickFormat = (number) => format('.4~g')(number);

const render = (data, total) => {
  const pieData = pie().value((d) =>
    d['Total Sales'].replace(/\$|,/g, ''),
  )(data);

  const colors = scaleOrdinal()
    .domain(data.map((d) => d['Product Category']))
    .range([
      '#FFC0CB',
      '#FFA07A',
      '#00CED1',
      '#98FB98',
      '#FFDAB9',
      '#ADD8E6',
      '#FFB6C1',
      '#F0E68C',
      '#E6E6FA',
      '#FF6347',
      '#F08080',
    ]);

  const segments = arc()
    .innerRadius(0)
    .outerRadius(170)
    .padAngle(0)
    .padRadius(0);

  const pieChart = svg
    .append('g')
    .attr(
      'transform',
      `translate(${width / 4},${height / 2})`,
    );

  const sections = pieChart
    .selectAll('path')
    .data(pieData)
    .enter()
    .append('path')
    .attr('d', segments)
    .attr('fill', (d) =>
      colors(d.data['Product Category']),
    );

  const legend = svg
    .append('g')
    .attr('class', 'legend')
    .attr('transform', 'translate(500,100)');

  const legendItems = legend
    .selectAll('.legendItem')
    .data(pieData)
    .enter()
    .append('g')
    .attr('class', 'legendItem')
    .attr('transform', (d, i) => `translate(0, ${i * 30})`);

  legendItems
    .append('rect')
    .attr('width', 20)
    .attr('height', 20)
    .attr('fill', (d) =>
      colors(d.data['Product Category']),
    );

  legendItems
    .append('text')
    .attr('x', 25)
    .attr('y', 15)
    .attr('class', 'legend_text')
    .text((d) => d.data['Product Category']);

  legendItems
    .append('text')
    .attr('x', 170)
    .attr('y', 15)
    .attr('class', 'legend_value')
    .text(
      (d) =>
        xAxisTickFormat(
          (
            (d.data['Total Sales'].replace(/\$|,/g, '') /
              total) *
            100
          ).toFixed(1),
        ) + '%',
    );

  legendItems
    .append('text')
    .attr('x', 250)
    .attr('y', 15)
    .attr('class', 'legend_value')
    .text(
      (d) =>
        xAxisTickFormat(
          d.data['Total Sales'].replace(/\$|,/g, '') / 1000,
        ) + 'K',
    );
};

const run = async () => {
  try {
    const data = await getData();
    const totalSales = data.reduce(
      (acc, curr) =>
        acc +
        parseFloat(
          curr['Total Sales'].replace(/\$|,/g, ''),
        ),
      0,
    );
    render(data, totalSales);
  } catch (error) {
    console.error('Error fetching data:', error);
  }
};

run();
